@@ -1,5 +1,5 @@
 package Anki::Import ;
-$Anki::Import::VERSION = '0.021';
+$Anki::Import::VERSION = '0.022';
 use strict;
 use warnings;
 use Cwd;
@@ -9,6 +9,10 @@ use Log::Log4perl::Shortcuts 0.011 qw(:all);
 use Exporter qw(import);
 our @EXPORT = qw(anki_import);
 
+# change log config to test for development for fine-tuned control over log output
+set_log_config('anki-import.cfg', __PACKAGE__);
+#set_log_config('test.cfg', __PACKAGE__);
+
 # set up variables
 my @lines;                # lines from source file
 my $line_count = 0;       # count processed lines to give more helpful error msg
@@ -18,9 +22,6 @@ my $ntype      = 'Basic'; # default note type
 my @notes      = ();      # array for storing notes
 my @autotags   = ();      # for storing automated tags
 
-set_log_config('anki-import.cfg', __PACKAGE__);
-#set_log_config('test.cfg', __PACKAGE__);
-
 # argument processing
 arg file => (
   isa => 'Str',
@@ -58,6 +59,9 @@ sub anki_import {
     logf('Aborting: No file passed to Anki::Import.');
   }
 
+  # set parent directory
+  my $pd = $args->{parent_dir};
+
   # set log level as appropriate
   if ($args->{verbose}) {
     set_log_level('info');
@@ -70,28 +74,26 @@ sub anki_import {
 
   # get and load the source file
   logi('Loading file');
-  my $path  = path($file);
-  logd($path);
+  my $path  = path($file); logd($path);
   if (!path($file)->exists) {
     logf("Aborting: Source file named '$file' does not exist.");
   };
-  @lines = $path->lines_utf8;
+  @lines = $path->lines_utf8; logi('Source file loaded.');
 
   # pad data with a blank line to make it easier to process
   push @lines, '';
 
   # do the stuff we came here for
-  validate_src_file();
-  logd(\@notes);
-
-  my $pd = $args->{parent_dir};
+  validate_src_file(); logd(\@notes);
   generate_importable_files($pd);
 
+  # print a success message
   unless ($args->{'quiet'}) {
     set_log_level('info');
     logi("Success! Your import files are in the $pd"
       . '/anki_import_files directory') unless $args->{quiet};
   }
+
   # fin
 }
 
@@ -153,6 +155,7 @@ sub slurp_note {
 
   # loop over lines in the note
   while (next_line()) {
+    logd($cline, 'cline');
     if ($cline =~ /^$|^\s+$/) {
       my @all_fields = @current_field;
       push (@note, \@all_fields) if @current_field;
@@ -168,19 +171,22 @@ sub slurp_note {
 }
 
 sub next_line {
-  return 0 if !@lines; # last line in file is always blank
+  return 0 if !@lines; # last line in file was made blank
   $lline = $cline;
   $cline = (shift @lines || '');
+
+  # do some cleanup
   chomp $cline;
+  $cline =~ s/\t/    /g; # replace tabs with spaces
+
   ++$line_count;
 }
 
-
 # functions for second pass parsing and formatting of source data
 # and creation of import files
 sub generate_importable_files {
-  my $pd = shift;
-  logi('Generating files for import');
+  my $pd = shift; logi('Generating files for import');
+
   my %filenames;
 
   # loop over notes
@@ -189,7 +195,7 @@ sub generate_importable_files {
 
     my $line = process_note($note->{note});
 
-    # write to our file out
+    # add our processed note to our data
     my $filename = $note->{ntype} . '_notes_import.txt';
     $filenames{$filename}{content} .= $line;
   }
@@ -202,57 +208,71 @@ sub generate_importable_files {
   }
 }
 
+# the meat of the matter
+# TODO: break up into shorter functions for readability
 sub process_note {
-  my $note = shift;
-  logd($note, 'note_2b_processed');
+  my $note = shift; logd($note, 'note_2b_processed');
 
-  my $new_autotags = 0;
   my @fields = ();
-  # loop over fields
+  my $new_autotags = 0; # flag raised if autotag line found
+
+  # loop over note fields
   foreach my $field (@$note) {
-    my $in_code = 0;   # tracks if we are preserving whitespace
+    my $ws_mode = 0;   # tracks if we are preserving whitespace
     my $field_out = '';
 
-    # loop over lines in field
-    my @lines = ('');
+    # loop over lines in field and process accordingly
+    my @lines = (''); # can't take a reference to nothing
     foreach my $line (@$field) {
-      my $last_line = \$lines[-1];
+      my $last_line = \$lines[-1]; # just to make it easier to type
 
-      # detect automated tags
+      # detect autotags
       logd($line);
-      if ($line =~ /^\+\s*$/ && !$in_code) {
+      if ($line =~ /^\+\s*$/ && !$ws_mode) {
         push @autotags, split (/\s+/, $$last_line);
         $new_autotags = 1;
       }
-      if ($line =~ /^\^\s*$/ && !$in_code) {
+      if ($line =~ /^\^\s*$/ && !$ws_mode) {
         @autotags = split (/\s+/, $$last_line);
         $new_autotags = 1;
         next;
       }
 
-      if ($line =~ /^`\s*$/ && !$in_code) {
+      # blanks lines not in non-whitespace mode
+      if ($line =~ /^`\s*$/ && !$ws_mode) {
         if ($$last_line && $$last_line !~ /^<br>+$/) {
           $$last_line .= '<br>';
         }
         next;
       }
-      if ($line =~ /^`{3,3}$/ && !$in_code) {
-        $in_code = 1;
+
+      # enter whitespace mode and adding appropriate HTML
+      if ($line =~ /^`{3,3}$/ && !$ws_mode) {
+        $ws_mode = 1;
+
+        # add a couple of blank lines to previous line
         if ($$last_line) {
           $$last_line .= '<br><br>';
         }
+
         $$last_line .= '<div style="text-align: left; font-family: courier; white-space: pre;">';
         next;
       }
 
-      # exit whitespace preservation mode
-      if ($line =~ /^`{3,3}$/ && $in_code) {
+      # exit whitespace mode, close out HTML, add blank lines
+      if ($line =~ /^`{3,3}$/ && $ws_mode) {
+        $ws_mode = 0;
         $$last_line .= "</div><br><br>";
-        $in_code = 0;
         next;
       }
-      if ($in_code) {
+
+      # handle lines differently based on if we are preserving whitespace
+      if ($ws_mode) {
         # escape characters in preserved text
+        if ($line =~ /^`\s*$/) {
+          $$last_line .= '<br>';
+          next;
+        }
         $line =~ s/(?<!\\)`/\\`/g;
         $line =~ s/(?<!\\)\*/\\*/g;
         $line =~ s/(?<!\\)%/\\%/g;
@@ -261,10 +281,20 @@ sub process_note {
         push @lines, $line;
       }
     }
-    # handle formatting codes in text, preserve escaped characters
+    logf('A set of backticks (```) is unmatched or you failed to backtick a'
+         . ' blank line inside of a backtick set. Please correct the source'
+         . ' file and try again. Run "perldoc Anki::Import" for more help.') if $ws_mode;
+
     logd($field_out, 'field_out');
+
+    shift @lines if !$lines[0];
     my $field = join ' ', @lines;
 
+    # clean up dangling breaks
+    $field =~ s/<br><\/div>/<\/div>/g;
+
+    # handle formatting codes in text, preserve escaped characters
+
     # backticked characters
     $field =~ s/(?<!\\)`(.*?)`/<span style="font-family: courier; weight: bold;">$1<\/span>/gm;
     $field =~ s/\\`/`/g;
@@ -282,25 +312,28 @@ sub process_note {
 
   }
 
-  # clean up extraneous characters at the end of the line
-
-  # handle autotagging TODO: Ugly, needs cleanup
+  # generate tag field
   if (@autotags && !$new_autotags) {
-    logd($note->[-1][0], 'lnote_elem');
-    my @note_tags = split (/\s+/, $fields[-1]);
-    logd(\@note_tags, 'raw_note_tags');
+
+    # get tags from tag field
+    my @note_tags = split (/\s+/, $fields[-1]); logd(\@note_tags, 'raw_note_tags');
     my @new_tags = ();
+
+    # add tags from tag field
     foreach my $note_tag (@note_tags) {
       my $in_autotags = grep { $_ eq $note_tag } @autotags;
       push @new_tags, $note_tag unless $in_autotags;
     }
+
+    # add autotags
     foreach my $autotag (@autotags) {
       my $discard_autotag = grep { $_ eq $autotag } @note_tags;
       push @new_tags, $autotag if !$discard_autotag;
     }
+
+    # add combined tags as a field
     logd(\@new_tags, 'new_tags');
     my $new_tags = join (' ', @new_tags);
-    #$new_tags =~ s/^\s+//;
     $fields[-1] = $new_tags;
   }
   $new_autotags = 0;
@@ -319,7 +352,6 @@ sub process_note {
   $out .= "\n";
 }
 
-
 1; # Magic true value
 # ABSTRACT: Anki note generation made easy.
 
@@ -333,7 +365,7 @@ Anki::Import - Anki note generation made easy.
 
 =head1 VERSION
 
-version 0.021
+version 0.022
 
 =head1 OVERVIEW
 
@@ -413,10 +445,10 @@ deck.
 
 Each note in the source file contains fields which should correspond to your
 existing note types in Anki. Individual notes in the source file are delineated
-by two or more blank lines. Fields are separated by a
-single blank line. Fields for each note should be in the same order as your
-Anki note types to make importing more automatic. All fields must have content
-or left intentionally blank.
+by two or more blank lines. Fields are separated by a single blank line. Fields
+for each note should be in the same order as your Anki note types to make
+importing more automatic. All fields must have content or left intentionally
+blank.
 
 To create an intionally blank field, add a single '`' (backtick) character on a
 line by itself with blank lines before and after the line with the single
@@ -424,10 +456,20 @@ backtick.
 
 See the L</Source file example> for more help.
 
-IMPORTANT: Save the source file as a plain text file with UTF-8 encoding. UTF-8
+=head2 Source file requirements and limitations
+
+=head3 Use UTF-8 encoding
+
+The source file should be a plain text file with UTF-8 encoding. UTF-8
 is likely the default encoding method for your editor but check your editor's
 settings and documentation for further details.
 
+=head3 Avoid tabs
+
+Since tab characters are used by Anki to split your fields, you should
+avoid relying on tab characters in your source file. Any tabs found in your
+source file will get converted to four spaces.
+
 =head3 Assigning notes to note types
 
 You can indicate which note type a note belongs to by preceding notes with a
@@ -487,7 +529,7 @@ generate simple lists. Study the example below for details.
 
 Note: Lines containing only whitespace characters are treated as blank lines.
 
-=head4 Source file example
+=head4 Example source file
 
 Below is an example of how to format a source data file. Note that the column on
 the right containing comments for this example are not permitted in an actual
@@ -554,12 +596,13 @@ source data file.
     What does this code do?              # Another less_basic question
 
     ```                                  # Preserve whitespace in a field with 3
-                                         # backticks on a single line
-    This_is_some_code {
-        print 'Whitespace will be        # Whitespace is preserved between the
-               preserved';               # sets of triple backticks
+    This_is_some_code {                  # backticks on a single line.
+    `                                    # You must still backtick blank lines
+        print 'Whitespace will be        # when preserving whitespace, however.
+               preserved';
+    `                                    # Another blank line.
     }
-    ```                                  # end whitespace preservation
+    ```                                  # End whitespace preservation
 
     This is %comma,delimted,text%        # Bullet lists with %item1,item2,item3%
 
@@ -584,8 +627,31 @@ source data file.
 
     Last anser
 
-    new_tag                              # We add a new_tag to our autotag list
-    +                                    # with the '+' sign.
+    add_this_tag_to_autotags             # We add a new_tag to our autotag list
+    +                                    # with the '+' sign by itself on a new
+                                         # line.
+
+=head3 Getting the most from C<Anki::Import>
+
+By itself, C<Anki::Import> will make it easier for you to format and
+input your notes especially if you do a lot of basic HTML formatting. However,
+the huge productivity gains of C<Anki::Import> can only be unlocked by getting
+proficient wih your text editor of choice.
+
+For example, you can generate templates for each of the note types you use to
+make data entry exceptionally painless. And with a text editor like vim, you
+can automate the generation of the formatting codes used by C<Anki::Import>
+and make Anki note creation joyful, or at least much less tedious.
+
+Teaching you how to use and optimize your text editor for C<Anki::Import> is
+well beyond the scope of this document. But if you take the time now and do the
+up front work of learning your text editor and tweaking it for use with
+C<Anki::Import>, you will save a lot of time in the long run.
+
+In the future, vim configurations and plugins for use with C<Anki::Import>
+may be released as they are developed to help you get going faster with vim.
+Unfortunately, other text editors cannot be supported as there are far too many
+and far too little time to get familiar with all their features.
 
 =head1 USAGE
 
@@ -643,7 +709,7 @@ want the function call to output a success message, use (C<--no-quiet>);
 =head1 INSTALLATION
 
 C<Anki::Import> is written in the Perl programming langauge. Therefore, you must
-have the Perl installed on your system. MacOS and *nix machines will have
+have Perl installed on your system. MacOS and *nix machines will have
 Perl already installed but the Windows operating system does not
 come pre-installed with Perl and so you may have to install it first before you
 can use C<Anki::Import>.
@@ -748,17 +814,6 @@ L<https://github.com/sdondley/Anki-Import>
 You can make new bug reports, and view existing ones, through the
 web interface at L<https://github.com/sdondley/Anki-Import/issues>.
 
-=head1 SEE ALSO
-
-=head2 Development status
-
-This module is currently in the beta stages and is actively supported and
-maintained. Suggestions for improvement are welcome. There are likely bugs
-with the text formatting in certain edge cases but it should work well for
-normal, intended use.
-
-L<Anki documentation|https://apps.ankiweb.net/docs/manual.html>
-
 =head1 AUTHOR
 
 Steve Dondley <s@dondley.com>
